# -*- coding: utf-8 -*-
from collective.mailchimp.testing import (
    COLLECTIVE_MAILCHIMP_INTEGRATION_TESTING,
)
from zope.component import getUtility

import unittest


class MailchimpLocatorIntegrationTest(unittest.TestCase):

    layer = COLLECTIVE_MAILCHIMP_INTEGRATION_TESTING

    def setUp(self):
        self.app = self.layer['app']
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        self.portal_url = self.portal.absolute_url()

    def test_mailchimp_locator_registration(self):
        from collective.mailchimp.interfaces import IMailchimpLocator

        self.assertTrue(getUtility(IMailchimpLocator))

    def test_mailchimp_locator_initialize_method(self):
        from collective.mailchimp.locator import MailchimpLocator

        locator = MailchimpLocator()
        self.assertEqual(locator.registry, None)
        self.assertEqual(locator.settings, None)
        self.assertEqual(locator.api_root, None)
        locator.initialize()
        self.assertTrue(locator.registry)
        self.assertTrue(locator.settings)
        self.assertTrue(locator.api_root)

    def test_mailchimp_locator_lists_method(self):
        from collective.mailchimp.locator import MailchimpLocator

        locator = MailchimpLocator()
        self.assertTrue(locator.lists())
        self.assertEqual(len(locator.lists()), 3)

    def test_mailchimp_locator_groups_method(self):
        from collective.mailchimp.locator import MailchimpLocator

        locator = MailchimpLocator()
        groups = locator.groups(list_id=u'57afe96172')
        self.assertTrue(groups)
        self.assertEqual(len(groups['categories']), 1)
        self.assertEqual(len(groups['interests']), 5)
        self.assertEqual(
            groups['interests'][0]['name'],
            u"Sometimes you just gotta 'spress yourself.",
        )

    def test_mailchimp_locator_update_subscriber_method(self):
        from collective.mailchimp.locator import MailchimpLocator

        locator = MailchimpLocator()
        member = locator.update_subscriber(
            '57afe96172',
            'freddy@freddiesjokes.com',
            interests={'a1e9f4b7f6': True},
        )
        self.assertTrue(member)
        self.assertEqual(member['interests']['a1e9f4b7f6'], True)

    def test_mailchimp_locator_updateCache_method(self):
        from collective.mailchimp.locator import MailchimpLocator

        locator = MailchimpLocator()
        locator.initialize()
        # These tests pass when we run this test method separately,
        # but fail when running all tests.  Skip them because they are
        # not what we really want to test here.
        #
        # self.assertEqual(locator.registry[locator.key_account], None)
        # self.assertEqual(locator.registry[locator.key_groups], None)
        # self.assertEqual(locator.registry[locator.key_lists], None)
        locator.updateCache()
        account = locator.registry[locator.key_account]
        self.assertTrue(isinstance(account, dict))
        self.assertEqual(account[u'account_id'], u'8d3a3db4d97663a9074efcc16')
        self.assertEqual(account[u'account_name'], u"Freddie's Jokes")
        groups = locator.registry[locator.key_groups]
        self.assertTrue(isinstance(groups, dict))
        group_keys = list(groups.keys())
        self.assertEqual(
            group_keys, ['f6257645gs', 'f6267645gs', '57afe96172']
        )
        self.assertEqual(
            sorted(list(groups[group_keys[0]].keys())),
            [
                '_links',
                'categories',
                'interests',
                'list_id',
                'total_items',
            ],
        )

        self.assertTrue(isinstance(locator.registry[locator.key_lists], tuple))
        self.assertEqual(len(locator.registry[locator.key_lists]), 3)
        # It does not complain when there is no api key
        locator.settings.api_key = None
        locator.updateCache()


def test_suite():
    return unittest.defaultTestLoader.loadTestsFromName(__name__)



# -*- coding: utf-8 -*-
from collective.mailchimp import _
from zope import schema
from zope.interface import Interface
from zope.interface import Invalid
from zope.interface import invariant
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary

import re


available_fields = SimpleVocabulary(
    [
        SimpleTerm(value=u'subscriber_list', title=_(u'Subscriber list')),
        SimpleTerm(value=u'email', title=_(u'E-Mail')),
    ]
)


class ICollectiveMailchimp(Interface):
    """Marker interface that defines a ZTK browser layer. We can reference
    this in the 'layer' attribute of ZCML <browser:* /> directives to ensure
    the relevant registration only takes effect when this theme is installed.

    The browser layer is installed via the browserlayer.xml GenericSetup
    import step.
    """


class NotAnEmailAddress(schema.ValidationError):
    __doc__ = _(u"Invalid email address")


check_email = re.compile(
    r"[a-zA-Z0-9._%-]+@([a-zA-Z0-9-]+.)*[a-zA-Z]{2,4}"
).match


def validate_email(value):
    if not check_email(value):
        raise NotAnEmailAddress(value)
    return True


class INewsletterSubscribe(Interface):

    email = schema.TextLine(
        title=_(u"Email address"),
        description=_(
            u"help_email", default=u"Please enter your email address."
        ),
        required=True,
        constraint=validate_email,
    )

    interest_groups = schema.Tuple(
        title=_(u"Interest groups"),
        description=_(u"help_interest_groups", default=u""),
        value_type=schema.Choice(
            vocabulary="collective.mailchimp.vocabularies.InterestGroups"
        ),
        required=False,
    )

    email_type = schema.Choice(
        title=_(u"Mail format"),
        vocabulary="collective.mailchimp.vocabularies.EmailType",
        default="html",
        required=False,
    )

    list_id = schema.TextLine(title=_(u"List ID"), required=False)


class INewsletterUnsubscribe(Interface):

    email = schema.TextLine(
        title=_(u"Email address"),
        description=_(
            u"help_email", default=u"Please enter your email address."
        ),
        required=True,
        constraint=validate_email,
    )

    list_id = schema.TextLine(title=_(u"List ID"), required=False)

    interest_groups = schema.Tuple(
        title=_(
            u'mailchimp_unsubscribe_interest_groups',
            default=u"Unsubscribe only form the following interest groups",
        ),
        description=_(
            u"mailchimp_help_unsubscribe_interest_groups", default=u""
        ),
        value_type=schema.Choice(
            vocabulary="collective.mailchimp.vocabularies.InterestGroups"
        ),
        required=False,
    )

    unsubscribe = schema.Bool(
        title=_(
            u'mailchimp_unsubscribe_newsletter',
            default=u"Unsubscribe from the complete newsletter",
        ),
        description=_(u'mailchimp_help_unsubscribe_newsletter', default=u''),
    )


class IMailchimpLocator(Interface):
    """Mailchimp API
    """

    def initialize():
        """Do a postmonkey with the API so your connected to mailchimp API"""

    def lists():
        """Return all available MailChimp lists.
        http://apidocs.mailchimp.com/api/rtfm/lists.func.php
        """

    def default_list_id():
        """Return the default list from the settings if it exists.
        Else return the first list available in your mailchimp account
        """

    def groups(list_id=None):
        """Return all available MailChimp interest groups.

        @id: the list id to connect to. e.g. u'a1346945ab'. Not the web_id!

        http://apidocs.mailchimp.com/api/rtfm/listinterestgroupings.func.php
        """

    def subscribe(list_id, email_address, merge_vars, email_type):
        """Do subscribe the email address to the list_id"""

    def account():
        """connect and return getAccountDetails"""


class IMailchimpSettings(Interface):
    """Global mailchimp settings. This describes records stored in the
    configuration registry and obtainable via plone.registry.
    """

    api_key = schema.TextLine(
        title=_(u"MailChimp API Key"),
        description=_(
            u"help_api_key",
            default=u"Enter in your MailChimp key here (.e.g. "
            + u"'8b785dcabe4b5aa24ef84201ea7dcded-us4'). Log into "
            + u"mailchimp.com, go to account -> extras -> API Keys & "
            + u"Authorized Apps and copy the API Key to this field.",
        ),
        default=u"",
        required=True,
    )

    email_type = schema.Choice(
        title=_(u"email_type", default=u"Email type"),
        description=_(
            u"help_email_type",
            default=u"Email type preference for the email (html, text, or "
            u"mobile defaults to html)",
        ),
        vocabulary="collective.mailchimp.vocabularies.EmailType",
        default="html",
        required=False,
    )

    email_type_is_optional = schema.Bool(
        title=_(u"email_type_is_optional", default=u"Email type is optional"),
        description=_(
            u"help_email_type_is_optional",
            default=u"Let users choose their email type preference in the "
            u"newsletter subscription form.",
        ),
        required=False,
        default=False,
    )

    default_list = schema.Choice(
        title=_(u"default_list", default=u"Default list"),
        description=_(
            u"help_default_list",
            default=u"Default list which is used in the @@newsletter view if "
            u"no list_id param is provided.",
        ),
        vocabulary="collective.mailchimp.vocabularies.AvailableLists",
        required=False,
    )

    max_lists_number = schema.Int(
        title=_(u"max_lists_number", default=u"Maximum lists number"),
        description=_(
            u"help_max_lists_number",
            default=u"Maximum number of lists you want to retrieve from "
            u"MailChimp API (maximum supported is 1000).",
        ),
        default=50,
        min=1,
        max=1000,
        required=True,
    )

    double_optin = schema.Bool(
        title=_(u"double_optin", default=u"Double opt-in"),
        description=_(
            u"help_double_optin",
            default=u"Flag to control whether a double opt-in confirmation "
            u"message is sent, defaults to true. Abusing this may "
            u"cause your account to be suspended.",
        ),
        required=False,
        default=True,
    )

    update_existing = schema.Bool(
        title=_(u"update_existing", default=u"Update existing"),
        description=_(
            u"help_update_existing",
            default=u"Flag to control whether existing subscribers should be "
            u"updated instead of throwing an error, defaults to false",
        ),
        required=False,
        default=False,
    )

    replace_interests = schema.Bool(
        title=_(u"replace_interests", default=u"Replace interests"),
        description=_(
            u"help_replace_interests",
            default=u"Flag to determine whether we replace the interest "
            u"groups with the groups provided or we add the provided"
            u"groups to the member's interest groups (optional, "
            u"defaults to true)",
        ),
        required=False,
        default=True,
    )

    send_welcome = schema.Bool(
        title=_(u"send_welcome", default=u"Send welcome"),
        description=_(
            u"help_send_welcome",
            default=u"If your double_optin is false and this is true, we "
            u"will send your lists Welcome Email if this subscribe "
            u"succeeds - this will *not* fire if we end up updating "
            u"an existing subscriber. If double_optin is true, this "
            u"has no effect. defaults to false.",
        ),
        required=False,
        default=False,
    )

    @invariant
    def valid_api_key(data):
        if len(data.api_key) == 0:
            return
        parts = data.api_key.split('-')
        if len(parts) != 2:
            raise Invalid(
                u"Your MailChimp API key is not valid. Please go "
                + u"to mailchimp.com and check your API key."
            )



# -*- coding: utf-8 -*-
from collective.mailchimp.interfaces import IMailchimpSettings
from mock import Mock
from mock import patch
from plone.app.testing import applyProfile
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer
from plone.registry.interfaces import IRegistry
from zope.component import getUtility
from zope.configuration import xmlconfig

import json
import os
import re


DUMMY_API_KEY = u"abc-us1"
TEST_DATA_DIR = os.path.join(os.path.dirname(__file__), 'tests', 'data')


class MockRequestsException(Exception):
    """Exception raised in the requests mock.

    This makes it easier to distinguish between exceptions of the
    original and the patched module.
    """


class MockRequests(object):
    """Class used as mock replacement for the requests module.
    """

    def __init__(self):
        self.calls = []

    @property
    def last_call(self):
        return self.calls[-1]

    @staticmethod
    def parse_arguments(*args, **kwargs):
        """Parse the arguments and return whatever we need.
        """
        if len(args) != 1:
            raise MockRequestsException(
                'Expected 1 argument, got {0}: {1}'.format(len(args), args)
            )
        # Check url
        url = args[0]
        mailchimp_url = 'api.mailchimp.com/3.0/'
        if mailchimp_url not in url:
            raise MockRequestsException(
                'Expected {0} in url {1}'.format(mailchimp_url, url)
            )
        # Remove mailchimp & query from url to get only the endpoint
        endpoint = url.split(mailchimp_url)[1].split("?")[0]
        # Check auth
        auth = kwargs.get('auth')
        if not auth:
            raise MockRequestsException('Expected auth in keyword arguments.')
        expected_auth = ('apikey', DUMMY_API_KEY)
        if (
            not isinstance(auth, tuple)
            or len(auth) != 2
            or expected_auth != auth
        ):
            raise MockRequestsException(
                'Expected auth {0} in keyword arguments. Got {1}'.format(
                    expected_auth, auth
                )
            )
        data = kwargs.get('data')
        # Load the data.
        if not data:
            raise MockRequestsException('Expected data in keyword arguments.')
        data = json.loads(data)
        return endpoint, data

    def post(self, *args, **kwargs):
        """This is a mock for post and get in the requests module.

        Return a json dictionary based on the end point.
        """
        endpoint, data = self.parse_arguments(*args, **kwargs)
        self.calls.append({'endpoint': endpoint, 'data': data})
        path = ''
        text = '{}'
        if not endpoint:
            path = os.path.join(TEST_DATA_DIR, 'account.json')
        elif endpoint == 'lists':
            path = os.path.join(TEST_DATA_DIR, 'lists.json')
        elif re.compile('lists/.*/interest-categories/.*/interests').match(
            endpoint
        ):
            path = os.path.join(TEST_DATA_DIR, 'interests.json')
        elif re.compile('lists/.*/interest-categories').match(endpoint):
            path = os.path.join(
                TEST_DATA_DIR, 'lists_interest_categories.json'
            )
        elif re.compile('lists/.*/members/[^/]*$').match(endpoint):
            path = os.path.join(TEST_DATA_DIR, 'member.json')
        else:
            pass
        if path:
            with open(path) as datafile:
                text = datafile.read()
        # Return mock response with text.
        return Mock(text=text)

    get = post
    put = post
    delete = post
    patch = post


class CollectiveMailchimp(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        # Create patcher to mock the requests module that is imported in
        # locator.py only.
        self.requests_patcher = patch(
            'collective.mailchimp.locator.requests', new_callable=MockRequests
        )
        # It looks like the patcher is started automatically, although the
        # documentation says you must start it explicitly.  Let's follow the
        # documentation.  Then we can also nicely stop it in tearDownZope.
        self.requests_patcher.start()

        # Load ZCML
        import collective.mailchimp

        xmlconfig.file(
            'configure.zcml',
            collective.mailchimp,
            context=configurationContext,
        )

    def setUpPloneSite(self, portal):
        applyProfile(portal, 'collective.mailchimp:default')

        registry = getUtility(IRegistry)
        mailchimp_settings = registry.forInterface(IMailchimpSettings)
        mailchimp_settings.api_key = DUMMY_API_KEY

    def tearDownZope(self, app):
        # Undo our requests patch.
        self.requests_patcher.stop()


COLLECTIVE_MAILCHIMP_FIXTURE = CollectiveMailchimp()
COLLECTIVE_MAILCHIMP_INTEGRATION_TESTING = IntegrationTesting(
    bases=(COLLECTIVE_MAILCHIMP_FIXTURE,),
    name="CollectiveMailchimp:Integration",
)
COLLECTIVE_MAILCHIMP_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(COLLECTIVE_MAILCHIMP_FIXTURE,),
    name="CollectiveMailchimp:Functional",
)



# -*- coding: utf-8 -*-
from Acquisition import aq_inner
from collective.mailchimp.browser.newsletter import NewsletterSubscriberForm
from collective.mailchimp.interfaces import INewsletterSubscribe
from plone.app.portlets.portlets import base
from plone.autoform.directives import widget
from plone.memoize.compress import xhtml_compress
from plone.memoize.instance import memoize
from plone.portlets.interfaces import IPortletDataProvider
from plone.z3cform import z2
from plone.z3cform.interfaces import IWrappedForm
from Products.CMFCore.utils import getToolByName
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from z3c.form import field
from z3c.form.browser.checkbox import CheckBoxFieldWidget
from z3c.form.interfaces import IFormLayer
from zope import schema
from zope.i18nmessageid import MessageFactory
from zope.interface import alsoProvides
from zope.interface import implementer


_ = MessageFactory('collective.mailchimp')


class IMailChimpPortlet(IPortletDataProvider):

    name = schema.TextLine(
        title=_(u'Title'), description=_(u'Title of the portlet')
    )

    widget(available_lists=CheckBoxFieldWidget)
    available_lists = schema.List(
        title=_(u'Available lists'),
        description=_(u'Select available lists to subscribe to.'),
        required=True,
        min_length=1,
        value_type=schema.Choice(
            source='collective.mailchimp.vocabularies.AvailableLists'
        ),
    )


@implementer(IMailChimpPortlet)
class Assignment(base.Assignment):

    def __init__(self, name=u'', available_lists=[]):
        self.name = name
        self.available_lists = available_lists

    @property
    def title(self):
        return _(u"MailChimp")


class Renderer(base.Renderer):
    fields = field.Fields(INewsletterSubscribe)
    _template = ViewPageTemplateFile('portlet.pt')
    form = NewsletterSubscriberForm

    def __init__(self, *args):
        base.Renderer.__init__(self, *args)

    def render(self):
        return xhtml_compress(self._template())

    @property
    def name(self):
        return self.data.name or _(u"Subscribe to newsletter")


    def update(self):
        super(Renderer, self).update()
        self.form = NewsletterSubscriberForm(aq_inner(self.context), self.request)
        alsoProvides(self.form, IWrappedForm)
        self.form.update()


class AddForm(base.AddForm):
    schema = IMailChimpPortlet
    label = _(u"Add MailChimp Portlet")
    description = _(
        u"This portlet displays a subscription form for a "
        + u"MailChimp newsletter."
    )

    def create(self, data):
        return Assignment(
            name=data.get('name', u''),
            available_lists=data.get('available_lists', []),
        )


class EditForm(base.EditForm):
    schema = IMailChimpPortlet
    label = _(u"Edit MailChimp Portlet")
    description = _(
        u"This portlet displays a subscription form for a "
        + u"MailChimp newsletter."
    )



# -*- coding: utf-8 -*-
from collective.mailchimp.browser.newsletter import NewsletterSubcriber
from collective.mailchimp.browser.newsletter import NewsletterSubscriberForm
from persistent import Persistent
from plone.z3cform.fieldsets import extensible
from zope import schema
from zope.annotation import factory
from zope.annotation.attribute import AttributeAnnotations
from zope.component import adapter
from zope.component import provideAdapter
from zope.interface import implementer
from zope.interface import Interface
from zope.publisher.interfaces.browser import IDefaultBrowserLayer


class IExtraBehavior(Interface):
    foo = schema.TextLine(title=u"Foo")
    bar = schema.TextLine(title=u"Bar")


@implementer(IExtraBehavior)
@adapter(NewsletterSubcriber)
class ExtraBehavior(Persistent):

    foo = u""
    bar = u""


ExtraBehavior = factory(ExtraBehavior)
provideAdapter(ExtraBehavior)
provideAdapter(AttributeAnnotations)


@adapter(Interface, IDefaultBrowserLayer, NewsletterSubscriberForm)
class ExtraBehaviorExtender(extensible.FormExtender):

    def __init__(self, context, request, form):
        self.context = context
        self.request = request
        self.form = form

    def update(self):
        if 'extra.foo' not in self.form.fields.keys():
            self.add(IExtraBehavior, prefix="extra")


provideAdapter(factory=ExtraBehaviorExtender, name=u"test.extender")  # noqa



# -*- coding: utf-8 -*-
from collective.mailchimp import _
from collective.mailchimp.interfaces import IMailchimpLocator
from collective.mailchimp.interfaces import IMailchimpSettings
from collective.mailchimp.interfaces import INewsletterSubscribe
from collective.mailchimp.interfaces import INewsletterUnsubscribe
from plone.app.layout.navigation.root import getNavigationRootObject
from plone.registry.interfaces import IRegistry
from plone.z3cform.fieldsets import extensible
from plone.z3cform.layout import wrap_form
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import safe_text
from Products.statusmessages.interfaces import IStatusMessage
from z3c.form import button
from z3c.form import field
from z3c.form import form
from z3c.form.browser.checkbox import CheckBoxFieldWidget
from z3c.form.browser.radio import RadioFieldWidget
from z3c.form.interfaces import ActionExecutionError
from z3c.form.interfaces import HIDDEN_MODE
from z3c.form.interfaces import WidgetActionExecutionError
from zExceptions import BadRequest
from zope.annotation.interfaces import IAttributeAnnotatable
from zope.component import getUtility
from zope.interface import implementer
from zope.interface import Invalid

import re


# Characters that are allowed in some fields.
# This tries to prevent hacking attempts that get us blocked.
CHARS_ALLOWED = re.compile(r"^[a-zA-Z0-9\-_\./@\+]*$").match


@implementer(INewsletterSubscribe, IAttributeAnnotatable)
class NewsletterSubcriber(object):

    title = u""


class NewsletterSubscriberForm(extensible.ExtensibleForm, form.Form):
    fields = field.Fields(INewsletterSubscribe)
    ignoreContext = True
    id = "newsletter-subscriber-form"
    label = _(u"Subscribe to newsletter")

    def updateActions(self):
        super(NewsletterSubscriberForm, self).updateActions()
        self.actions['subscribe'].addClass('context')

    def updateFields(self):
        super(NewsletterSubscriberForm, self).updateFields()
        self.fields['interest_groups'].widgetFactory = CheckBoxFieldWidget
        self.fields['email_type'].widgetFactory = RadioFieldWidget

    def updateWidgets(self):
        super(NewsletterSubscriberForm, self).updateWidgets()
        widgets = self.widgets
        registry = getUtility(IRegistry)
        try:
            self.mailchimp_settings = registry.forInterface(IMailchimpSettings)
        except KeyError:
            self.mailchimp_settings = None
            return
        self.mailchimp = getUtility(IMailchimpLocator)

        # Show/hide mail format option widget
        if not self.mailchimp_settings.email_type_is_optional:
            widgets['email_type'].mode = HIDDEN_MODE

        # Retrieve the list id either from the request/form or fall back to
        # the default_list setting.
        list_id = self.context.REQUEST.get('list_id')
        list_id = list_id or self.request.form.get('form.widgets.list_id')
        list_id = list_id or self.mailchimp_settings.default_list
        widgets['list_id'].mode = HIDDEN_MODE
        widgets['list_id'].value = list_id

        # Show/hide interest_groups widget
        self.available_interest_groups = self.mailchimp.groups(list_id=list_id)
        if not self.available_interest_groups or \
           self.available_interest_groups.get('total_items') == 0:
            widgets['interest_groups'].mode = HIDDEN_MODE

        preselected_groups = self.request.get('preselected_group', [])
        for group_index in preselected_groups:
            group_index = int(group_index)
            widgets['interest_groups'].items[group_index]['checked'] = True

    @button.buttonAndHandler(
        _(u"subscribe_to_newsletter_button", default=u"Subscribe"),
        name='subscribe',
    )
    def handleApply(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return

        # Retrieve list_id either from a hidden field in the form or fetch
        # the first list from mailchimp.
        list_id = data.get('list_id')
        if list_id and not CHARS_ALLOWED(list_id):
            # I saw a hacker adapt the hidden field:
            # "actual-id' UNION ALL SELECT NULL,NULL,NULL,NULL#"
            # Currently the list id is hexadecimal, but I suppose
            # we can't rely on that.
            raise BadRequest("list_id has disallowed characters")

        list_id = list_id or self.mailchimp.default_list_id()
        # list_id has to be updated for merge_fields=data to work if None
        if "list_id" in data and not data.get('list_id'):
            data['list_id'] = list_id

        # interest groups
        interests = {}
        interest_groups = data.pop('interest_groups', [])
        if self.available_interest_groups and interest_groups:
            interest_groups = map(safe_text, interest_groups)
            for group in interest_groups:
                if not CHARS_ALLOWED(group):
                    raise BadRequest("interest_groups has disallowed characters")
            # Create dictionary with as keys the interest groups, and as
            # values always True.
            interests = dict.fromkeys(interest_groups, True)
        # Use email_type if one is provided by the form, if not choose the
        # default email type from the control panel settings.
        email_type = data.get('email_type')
        if email_type and not CHARS_ALLOWED(email_type):
            raise BadRequest("email_type has disallowed characters")

        email_type = email_type or self.mailchimp_settings.email_type

        # Subscribe to MailChimp list
        try:
            self.mailchimp.subscribe(
                list_id=list_id,
                email_address=data['email'],
                email_type=email_type,
                interests=interests,
                merge_fields=data,
            )
        except Exception as error:
            return self.handle_error(error, data)

        if self.mailchimp_settings.double_optin:
            message = _(
                u"We have to confirm your email address. In order to "
                + u"finish the newsletter subscription, click on the link "
                + u"inside the email we just send you."
            )
        else:
            message = _(
                u"You have been subscribed to our newsletter succesfully."
            )

        IStatusMessage(self.context.REQUEST).addStatusMessage(
            message, type="info"
        )
        portal = getToolByName(self.context, 'portal_url').getPortalObject()
        site = getNavigationRootObject(self.context, portal)
        self.request.response.redirect(site.absolute_url())

    def handle_error(self, error, data):
        # Current api v3 documentation only lists errors in the 400 and 500
        # range.  The 400 code can mean a lot of things...
        # Also, not all exceptions have a code.
        error_code = getattr(error, "code", 500)
        if error_code == 400:
            error_msg = _(
                u"mailchimp_error_msg_already_subscribed",
                default=u"Could not subscribe to newsletter. "
                u"Either the email '${email}' is already subscribed "
                u"or something else is wrong. Try again later.",
                mapping={u"email": data['email']},
            )
            translated_error_msg = self.context.translate(error_msg)
            raise WidgetActionExecutionError(
                'email', Invalid(translated_error_msg)
            )
        elif error_code == 220:
            error_msg = _(
                u"mailchimp_error_msg_banned",
                default=u"Could not subscribe to newsletter. "
                u"The email '${email}' has been banned.",
                mapping={u"email": data['email']},
            )
            translated_error_msg = self.context.translate(error_msg)
            raise WidgetActionExecutionError(
                'email', Invalid(translated_error_msg)
            )
        else:
            error_msg = _(
                u"mailchimp_error_msg",
                default=u"Could not subscribe to newsletter. "
                u"Please contact the site administrator: "
                u"'${error}'",
                mapping={u"error": error},
            )
            translated_error_msg = self.context.translate(error_msg)
            raise ActionExecutionError(Invalid(translated_error_msg))


NewsletterView = wrap_form(NewsletterSubscriberForm)  # noqa


class UnsubscribeNewsletterForm(extensible.ExtensibleForm, form.Form):

    fields = field.Fields(INewsletterUnsubscribe)
    ignoreContext = True
    id = "newsletter-unsubscriber-form"
    label = _(
        u'mailchimp_unsubscribe_newsletter_form_title',
        default=u"Unsubscribe from newsletter",
    )

    description = _(
        u'mailchimp_unsubscribe_newsletter_form_description', default=''
    )

    def updateFields(self):
        super(UnsubscribeNewsletterForm, self).updateFields()
        self.fields['interest_groups'].widgetFactory = CheckBoxFieldWidget

    def updateWidgets(self):
        super(UnsubscribeNewsletterForm, self).updateWidgets()
        registry = getUtility(IRegistry)
        self.mailchimp_settings = registry.forInterface(IMailchimpSettings)
        self.mailchimp = getUtility(IMailchimpLocator)

        # Set a given email address
        if self.request.get('email'):
            self.widgets['email'].value = self.request['email']

        # Retrieve the list id either from the request/form or fall back to
        # the default_list setting.
        list_id = self.context.REQUEST.get('list_id')
        list_id = list_id or self.request.form.get('form.widgets.list_id')
        list_id = list_id or self.mailchimp_settings.default_list
        self.widgets['list_id'].mode = HIDDEN_MODE
        self.widgets['list_id'].value = list_id

        # Show/hide interest_groups widget
        self.available_interest_groups = self.mailchimp.groups(list_id=list_id)
        if not self.available_interest_groups:
            self.widgets['interest_groups'].mode = HIDDEN_MODE

    @button.buttonAndHandler(
        _(u"unsubscribe_newsletter_button", default=u"Unsubscribe"),
        name='unsubscribe',
    )
    def handle_unsubscribe(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return

        list_id = data.get('list_id') or self.mailchimp.default_list_id()
        email = data['email']

        update_data = {}
        if data.get('unsubscribe'):
            update_data['status'] = 'unsubscribed'
        else:
            interest_groups = {}
            if data.get('interest_groups'):
                for group in data.get('interest_groups', []):
                    interest_groups[group] = False
                update_data['interests'] = interest_groups

        try:
            self.mailchimp.update_subscriber(
                list_id, email_address=email, **update_data
            )
        except Exception as error:
            if getattr(error, "code", 500) != 404:
                # If a subscriber did not exist we don't want to announce
                # it. Treat only != 404 as an error.
                IStatusMessage(self.request).addStatusMessage(
                    _(
                        u'mailchimp_unsubscribe_error_msg',
                        default=u'We could not unsubscribe you from '
                        u'the newsletter. '
                        u"Please contact the site administrator: "
                        u"'${error}'",
                        mapping={u"error": error},
                    ),
                    type="info",
                )

        IStatusMessage(self.request).addStatusMessage(
            _(
                u'mailchimp_unsubscribed_msg',
                default=(
                    u'Thank you. You have been unsubscribed from '
                    u'the Newsletter.'
                ),
            ),
            type="info",
        )

        portal = getToolByName(self.context, 'portal_url').getPortalObject()
        site = getNavigationRootObject(self.context, portal)
        self.request.response.redirect(site.absolute_url())


UnsubscribeNewsletterView = wrap_form(UnsubscribeNewsletterForm)



# -*- coding: utf-8 -*-
import unittest


class TestExceptions(unittest.TestCase):
    def test_SerializationError_captures_obj(self):
        from ..exceptions import SerializationError

        dummy_obj = object()
        error = SerializationError(dummy_obj)
        self.assertEqual(error.obj, dummy_obj)
        self.assertIn(dummy_obj.__str__(), error.__str__())

    def test_DeserializationError_captures_obj(self):
        from ..exceptions import DeserializationError

        dummy_obj = object()
        error = DeserializationError(dummy_obj)
        self.assertEqual(error.obj, dummy_obj)
        self.assertIn(dummy_obj.__str__(), error.__str__())

    def test_PostRequestError_captures_exc(self):
        from ..exceptions import PostRequestError

        exc = Exception()
        caught = PostRequestError(exc)
        self.assertEqual(caught.exc, exc)
        self.assertIn(exc.__str__(), caught.__str__())

    def test_MailChimpException_attrs(self):
        from ..exceptions import MailChimpException

        code = -90
        detail = 'Method fake_method is not exported by this server'
        errors = [
            {
                "field": "interests",
                "message": "Schema describes object, array found instead",
            }
        ]
        exc = MailChimpException(code, detail, errors)
        self.assertEqual(exc.code, code)
        self.assertEqual(exc.detail, detail)
        self.assertEqual(exc.errors, errors)
        self.assertIn(detail, exc.__str__())
        self.assertIn('interests', exc.__str__())




import unittest


class TestUnit(unittest.TestCase):
    def test_list_id_allowed(self):
        from collective.mailchimp.browser.newsletter import CHARS_ALLOWED as allowed

        self.assertTrue(allowed("abcde12345"))
        self.assertTrue(allowed("12345ABCDE"))
        self.assertTrue(allowed("4f17c3b08a"))
        self.assertTrue(allowed("html"))
        self.assertTrue(allowed("plain-text"))
        self.assertTrue(allowed("text/plain"))
        self.assertTrue(allowed("maurits@example.org"))
        self.assertTrue(allowed("maurits+test@example.org"))
        self.assertFalse(allowed("abcde1234' hack em"))
        self.assertFalse(allowed("abcde1234;"))
        self.assertFalse(allowed("abcde1234?"))



# -*- coding: utf-8 -*-
from collective.mailchimp.interfaces import IMailchimpSettings
from plone.app.upgrade.utils import loadMigrationProfile
from plone.registry.interfaces import IRegistry
from Products.CMFCore.utils import getToolByName
from zope.component import getUtility


PROFILE_ID = "profile-collective.mailchimp:default"


def update_registry(context):
    registry = getUtility(IRegistry)
    registry.registerInterface(IMailchimpSettings)


def install_mailchimp_stylesheet(context):
    csstool = getToolByName(context, 'portal_css', None)
    if csstool is None:
        return
    stylesheet_id = (
        '++resource++collective.mailchimp.stylesheets/mailchimp.css'
    )
    if stylesheet_id not in csstool.getResourceIds():
        csstool.manage_addStylesheet(
            id=stylesheet_id,
            rel='stylesheet',
            rendering='link',
            enabled=True,
            cookable=True,
        )


def install_mailchimp_cache(context):
    setup = getToolByName(context, 'portal_setup')
    setup.runImportStepFromProfile(PROFILE_ID, 'plone.app.registry')


def reload_profile(context):
    loadMigrationProfile(context, 'profile-collective.mailchimp:default')


def add_max_lists_number_setting(context):
    registry = getUtility(IRegistry)
    # we need to omit other fields because vocabularies (ex: available_lists)
    # use our new (not existing) field and cause traceback
    registry.registerInterface(
        IMailchimpSettings,
        omit=(
            'api_key',
            'default_list',
            'double_optin',
            'email_type',
            'email_type_is_optional',
            'replace_interests',
            'send_welcome',
            'update_existing',
        ),
    )



# -*- coding: utf-8 -*-
from ..exceptions import MailChimpException
from ..exceptions import PostRequestError
from collective.mailchimp import _
from collective.mailchimp.interfaces import IMailchimpLocator
from collective.mailchimp.interfaces import IMailchimpSettings
from plone.app.registry.browser import controlpanel
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.Five.browser import BrowserView
from z3c.form.interfaces import WidgetActionExecutionError
from zope.component import getUtility
from zope.interface import alsoProvides
from zope.interface import Invalid


try:
    from plone.protect.interfaces import IDisableCSRFProtection
except ImportError:
    # BBB for old plone.protect, default until at least Plone 4.3.7.
    IDisableCSRFProtection = None


class MailchimpSettingsEditForm(controlpanel.RegistryEditForm):

    schema = IMailchimpSettings
    label = _(u"MailChimp settings")
    description = _(u"""""")

    def update(self):
        self.updateCache()
        super(MailchimpSettingsEditForm, self).update()

    def updateFields(self):
        super(MailchimpSettingsEditForm, self).updateFields()

    def updateWidgets(self):
        super(MailchimpSettingsEditForm, self).updateWidgets()

    def updateCache(self):
        if IDisableCSRFProtection is not None:
            alsoProvides(self.request, IDisableCSRFProtection)
        mailchimp = getUtility(IMailchimpLocator)
        mailchimp.updateCache()


class MailchimpSettingsControlPanel(controlpanel.ControlPanelFormWrapper):
    form = MailchimpSettingsEditForm
    index = ViewPageTemplateFile('controlpanel.pt')


class MailchimpData(BrowserView):
    label = _(u"MailChimp data")

    def mailchimp_account(self):
        if IDisableCSRFProtection is not None:
            alsoProvides(self.request, IDisableCSRFProtection)
        mailchimp = getUtility(IMailchimpLocator)
        try:
            return mailchimp.account()
        except PostRequestError:
            return []
        except MailChimpException as error:
            raise WidgetActionExecutionError(
                Invalid(
                    u"Could not fetch account details from MailChimp. "
                    + u"Please check your MailChimp API key: %s" % error
                )
            )

    def available_lists(self):
        if IDisableCSRFProtection is not None:
            alsoProvides(self.request, IDisableCSRFProtection)
        mailchimp = getUtility(IMailchimpLocator)
        try:
            return mailchimp.lists()
        except MailChimpException as error:
            raise WidgetActionExecutionError(
                Invalid(
                    u"Could not fetch available lists from MailChimp. "
                    + u"Please check your MailChimp API key: %s" % error
                )
            )



# -*- coding: utf-8 -*-
from collective.mailchimp.testing import (
    COLLECTIVE_MAILCHIMP_INTEGRATION_TESTING,
)
from collective.mailchimp.testing import DUMMY_API_KEY
from collective.mailchimp.testing import MockRequestsException
from collective.mailchimp.testing import TEST_DATA_DIR

import os
import unittest


class MockRequestsTests(unittest.TestCase):
    """Test the mocking we do to the requests module.

    We only mock the requests module that is imported in locator.py.
    """

    layer = COLLECTIVE_MAILCHIMP_INTEGRATION_TESTING

    def test_locator_requests_error(self):
        # Test that our mock exceptions are raised.  We try to raise them all
        # here, but it seems overkill to test the messages of the exceptions.
        from collective.mailchimp.locator import requests

        self.assertRaises(MockRequestsException, requests.get)
        self.assertRaises(MockRequestsException, requests.get, '')
        self.assertRaises(
            MockRequestsException, requests.get, 'http://website.unavailable'
        )
        url = 'https://api.mailchimp.com/3.0/'
        self.assertRaises(MockRequestsException, requests.get, url)
        auth = ('apikey', DUMMY_API_KEY)
        self.assertRaises(MockRequestsException, requests.get, url, auth=auth)
        self.assertRaises(
            MockRequestsException, requests.get, url, auth=auth, data=''
        )
        self.assertRaises(
            ValueError, requests.get, url, auth=auth, data='{bad json'
        )

    def test_locator_requests_success(self):
        # Test that a correct api request is answered with our mock data.
        from collective.mailchimp.locator import requests

        url = 'https://api.mailchimp.com/3.0/'
        auth = ('apikey', DUMMY_API_KEY)
        response = requests.get(url, auth=auth, data='{}')
        path = os.path.join(TEST_DATA_DIR, 'account.json')
        with open(path) as datafile:
            expected_text = datafile.read()
        self.assertEqual(response.text, expected_text)

    def test_standard_requests_error(self):
        # Test that the normal exceptions are raised.
        import requests

        self.assertRaises(TypeError, requests.get)
        self.assertRaises(ValueError, requests.get, '')
        self.assertRaises(
            requests.exceptions.ConnectionError,
            requests.get,
            'http://website.unavailable',
        )



# -*- coding: utf-8 -*-


class SerializationError(Exception):
    """ Raised if a method call contains parameters that cannot be serialized to
    JSON. The object that caused the error is made available via the ``obj``
    attribute.
    """

    def __init__(self, obj):
        self.obj = obj

    def __str__(self):
        return '%s could not be serialized to json' % self.obj


class DeserializationError(Exception):
    """ Raised if MailChimp responds with anything other than valid JSON.
    We use MailChimp's JSON API exclusively so it is unlikely
    that this will be raised. The response that caused the error is
    made available via the ``obj`` attribute.
    """

    def __init__(self, obj):
        self.obj = obj

    def __str__(self):
        return '%s could not be deserialized' % self.obj


class PostRequestError(Exception):
    """ If any exception is made during the POST request to MailChimp's server,
    `PostRequestError` will be raised. It wraps the underlying exception
    object and makes it available via the ``exc`` attribute.
    """

    def __init__(self, exc):
        self.exc = exc

    def __str__(self):
        return 'Caught exc "%s" with error "%s"' % (type(self.exc), self.exc)


class MailChimpException(Exception):
    """ If MailChimp returns an exception code as part of their response,
    this exception will be raised. Contains the unmodified ``code`` (int) and
    ``error`` (unicode) attributes returned by MailChimp.
    """

    def __init__(self, code, detail, errors=''):
        self.code = code
        self.detail = detail
        self.errors = errors

    def __str__(self):
        return 'MailChimp error code {0}: "{1}"\n{2}'.format(
            self.code, self.detail, self.errors
        )


class AkamaiException(MailChimpException):
    """Akamai is an provider upstream for MailChimp.

    It may give 503 code if you are blocked.  Sample:

    {'type': 'akamai_error_message', 'title': 'akamai_503', 'status': 503,
     'ref_no': 'Reference Number: ...'}
    """

    def __init__(self, code, detail, errors=''):
        self.code = code
        self.detail = detail
        self.errors = errors

    def __str__(self):
        return 'Akamai error code {0}: "{1}"\n{2}'.format(
            self.code, self.detail, self.errors
        )



# -*- coding: utf-8 -*-
from collective.mailchimp.testing import (
    COLLECTIVE_MAILCHIMP_INTEGRATION_TESTING,
)
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import get_installer
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID

import unittest


class TestSetup(unittest.TestCase):

    layer = COLLECTIVE_MAILCHIMP_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']

    def test_browserlayer_available(self):
        from plone.browserlayer import utils
        from collective.mailchimp.interfaces import ICollectiveMailchimp

        self.assertTrue(ICollectiveMailchimp in utils.registered_layers())

    def test_mailchimp_resource_bundle_available(self):
        from zope.component import getUtility
        from plone.registry.interfaces import IRegistry
        from Products.CMFPlone.interfaces import IResourceRegistry

        reg = getUtility(IRegistry)
        resources = reg.collectionOfInterface(
            IResourceRegistry, prefix="plone.bundles", check=False
        )
        key = 'collective.mailchimp'
        self.assertIn(key, resources.keys())

    def test_mailchimp_css_enabled(self):
        portal_url = self.portal.absolute_url()
        css = "++resource++collective.mailchimp.stylesheets/mailchimp.css"
        # render the homepage
        html = self.portal()
        self.assertIn(css, html)


class TestUninstall(unittest.TestCase):

    layer = COLLECTIVE_MAILCHIMP_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.installer = get_installer(self.portal, self.layer['request'])
        roles_before = api.user.get_roles(TEST_USER_ID)
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.installer.uninstall_product('collective.mailchimp')
        setRoles(self.portal, TEST_USER_ID, roles_before)

    def test_product_uninstalled(self):
        """Test if collective.mailchimp is cleanly uninstalled."""
        self.assertFalse(
            self.installer.is_product_installed('collective.mailchimp')
        )

    def test_browserlayer_removed(self):
        """Test that ICollectiveMailchimp is removed."""
        from collective.mailchimp.interfaces import ICollectiveMailchimp
        from plone.browserlayer import utils

        self.assertNotIn(ICollectiveMailchimp, utils.registered_layers())

    def test_mailchimp_resource_bundle_removed(self):
        from zope.component import getUtility
        from plone.registry.interfaces import IRegistry
        from Products.CMFPlone.interfaces import IResourceRegistry

        reg = getUtility(IRegistry)
        resources = reg.collectionOfInterface(
            IResourceRegistry, prefix="plone.bundles", check=False
        )
        key = 'collective.mailchimp'
        self.assertNotIn(key, resources.keys())

    def test_mailchimp_css_disabled(self):
        portal_url = self.portal.absolute_url()
        css = "++resource++collective.mailchimp.stylesheets/mailchimp.css"
        # render the homepage
        html = self.portal()
        self.assertNotIn(css, html)


def test_suite():
    return unittest.defaultTestLoader.loadTestsFromName(__name__)



# -*- coding: utf-8 -*-
"""Installer for the collective.mailchimp package."""

from setuptools import find_packages
from setuptools import setup


long_description = (
    open('README.rst').read()
    + '\n'
    + 'Contributors\n'
    + '============\n'
    + '\n'
    + open('CONTRIBUTORS.rst').read()
    + '\n'
    + open('CHANGES.rst').read()
    + '\n'
)


setup(
    name='collective.mailchimp',
    version='4.0.0a2',
    description="MailChimp integration for Plone.",
    long_description=long_description,
    # Get more from https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        "Environment :: Web Environment",
        "Development Status :: 5 - Production/Stable",
        "Framework :: Plone",
        "Framework :: Plone :: Addon",
        "Framework :: Plone :: 5.2",
        "Framework :: Plone :: 6.0",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Framework :: Plone",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
    ],
    keywords='Python Plone MailChimp Mail Newsletter',
    author='kitconcept GmbH (Timo Stollenwerk)',
    author_email='stollenwerk@kitconcept.com',
    url='https://pypi.org/project/collective.mailchimp/',
    license='GPL version 2',
    packages=find_packages('src', exclude=['ez_setup']),
    namespace_packages=['collective'],
    package_dir={'': 'src'},
    include_package_data=True,
    zip_safe=False,
    python_requires=">=3.8",
    install_requires=[
        'setuptools',
        'Products.CMFPlone',
        'plone.app.portlets',
        'plone.app.registry',
        'plone.app.upgrade',
        'requests',
    ],
    extras_require={'test': ['mock', 'plone.app.testing', 'plone.api']},
    entry_points="""
    [z3c.autoinclude.plugin]
    target = plone
    """,
)



# -*- coding: utf-8 -*-
from collective.mailchimp.interfaces import IMailchimpSettings
from collective.mailchimp.testing import (
    COLLECTIVE_MAILCHIMP_INTEGRATION_TESTING,
)
from plone.app.testing import logout
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.registry import Registry
from plone.testing.z2 import Browser
from Products.CMFCore.utils import getToolByName
from zope.component import getMultiAdapter

import unittest


class TestMailchimpSettingsControlPanel(unittest.TestCase):

    layer = COLLECTIVE_MAILCHIMP_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        self.registry = Registry()
        self.registry.registerInterface(IMailchimpSettings)

    def test_mailchimp_controlpanel_view(self):
        view = getMultiAdapter(
            (self.portal, self.portal.REQUEST), name="mailchimp-settings"
        )
        # view = view.__of__(self.portal)
        self.assertTrue(view())

    def test_mailchimp_controlpanel_view_protected(self):
        from AccessControl import Unauthorized

        logout()
        self.assertRaises(
            Unauthorized,
            self.portal.restrictedTraverse,
            '@@mailchimp-settings',
        )

    def test_mailchimp_in_controlpanel(self):
        self.controlpanel = getToolByName(self.portal, "portal_controlpanel")
        self.assertTrue(
            'mailchimp'
            in [
                a.getAction(self)['id']
                for a in self.controlpanel.listActions()
            ]
        )

    def test_record_api_key(self):
        record = self.registry.records[
            'collective.mailchimp.interfaces.IMailchimpSettings.api_key'
        ]
        self.assertTrue('api_key' in IMailchimpSettings)
        self.assertEqual(record.value, u"")


class ControlpanelFunctionalTest(unittest.TestCase):

    layer = COLLECTIVE_MAILCHIMP_INTEGRATION_TESTING

    def setUp(self):
        app = self.layer['app']
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        self.portal_url = self.portal.absolute_url()
        self.browser = Browser(app)
        self.browser.handleErrors = False
        self.browser.addHeader(
            'Authorization',
            'Basic %s:%s' % (SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
        )

    def test_empty_form(self):
        self.browser.open("%s/mailchimp-settings" % self.portal_url)
        self.assertTrue("MailChimp settings" in self.browser.contents)



# -*- coding: utf-8 -*-
from collective.mailchimp.interfaces import IMailchimpLocator
from zope.component import getUtility
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary


def available_lists(context):
    mailchimp = getUtility(IMailchimpLocator)
    lists = mailchimp.lists()
    if not lists:
        return SimpleVocabulary([])
    return SimpleVocabulary(
        [SimpleTerm(value=li['id'], title=li['name']) for li in lists]
    )


def interest_groups(context):
    mailchimp = getUtility(IMailchimpLocator)
    if 'list_id' in context.REQUEST:
        # list_id URL parameter
        list_id = context.REQUEST['list_id']
    elif 'form.widgets.list_id' in context.REQUEST.form:
        # list_id form parameter
        list_id = context.REQUEST.form['form.widgets.list_id']
    else:
        # If no id param has been provided just take the first list.
        list_id = mailchimp.default_list_id()
        if not list_id:
            return SimpleVocabulary([])
    groups = mailchimp.groups(list_id=list_id)
    if not groups:
        return SimpleVocabulary([])
    # Each category has a list of options/groups/interests.  We only support
    # one interest category per list.  We take the first one.  We have stored
    # this in the interests key.
    interests = groups.get('interests')
    if not interests:
        return SimpleVocabulary([])
    return SimpleVocabulary(
        [
            SimpleTerm(value=group['id'].encode("utf-8"), title=group['name'])
            for group in interests
        ]
    )


def email_type(context):
    terms = []
    terms.append(SimpleTerm(value='text', token='text', title='Plain text'))
    terms.append(SimpleTerm(value='html', token='html', title='HTML'))
    return SimpleVocabulary(terms)



# -*- coding: utf-8 -*-
from .exceptions import AkamaiException
from .exceptions import DeserializationError
from .exceptions import MailChimpException
from .exceptions import PostRequestError
from .exceptions import SerializationError
from collective.mailchimp.interfaces import IMailchimpLocator
from collective.mailchimp.interfaces import IMailchimpSettings
from plone.registry.interfaces import IRegistry
from zope.component import getUtility
from zope.component.hooks import getSite
from zope.interface import implementer

import hashlib
import json
import logging
import requests
import six.moves


try:
    import urlparse
    from urllib import urlencode
except ImportError:
    from urllib import parse as urlparse
    from urllib.parse import urlencode

_marker = object()
logger = logging.getLogger('collective.mailchimp')


@implementer(IMailchimpLocator)
class MailchimpLocator(object):
    """Utility for MailChimp API calls.
    """

    key_account = "collective.mailchimp.cache.account"
    key_groups = "collective.mailchimp.cache.groups"
    key_lists = "collective.mailchimp.cache.lists"

    def __init__(self, settings={}):
        """ Use settings if provided

        Note that the __init__ method is only called once at startup.
        settings can only be passed when you directly instantiate a
        MailchimpLocator, for example in tests.
        """
        self.registry = None
        self.settings = None
        self.api_root = None
        if settings:
            self.settings = settings
        self.site_path = ""

    def initialize(self):
        """ Load settings from registry and construct api root"""
        site_path = "/".join(getSite().getPhysicalPath())
        if self.site_path != site_path:
            # Settings are for a different Plone Site.
            # See https://github.com/collective/collective.mailchimp/issues/31
            self.site_path = site_path
            self.registry = None
            self.settings = None
            self.api_root = None
        if self.registry is None:
            self.registry = getUtility(IRegistry)
        if self.settings is None:
            try:
                self.settings = self.registry.forInterface(IMailchimpSettings)
            except KeyError:
                return
        self.apikey = self.settings.api_key
        if not self.apikey:
            return
        parts = self.apikey.split('-')
        if not len(parts) > 1:
            # bad api key, allow to fix
            return
        self.shard = parts[1]
        self.api_root = "https://" + self.shard + ".api.mailchimp.com/3.0/"

    def _serialize_payload(self, payload):
        """ Merges any default parameters from ``self.params`` with the
        ``payload`` (giving the ``payload`` precedence) and attempts to
        serialize the resulting ``dict`` to JSON.
        Note: MailChimp expects the JSON string to be quoted (their docs
        call it "URL encoded", but python's ``urllib`` calls it "quote").
        Raises a ``SerializationError`` if data cannot be serialized.
        """
        params = self.params.copy()
        params.update(payload)
        try:
            jsonstr = json.dumps(params)
            serialized = six.moves.urllib.parse.quote(jsonstr)
            return serialized
        except TypeError:
            raise SerializationError(payload)

    def _deserialize_response(self, text):
        """ Attempt to deserialize a JSON response from the server."""
        try:
            deserialized = json.loads(text)
        except ValueError:
            raise DeserializationError(text)
        self._fail_if_mailchimp_exc(deserialized)
        return deserialized

    def _fail_if_mailchimp_exc(self, response):
        """ If MailChimp returns an exception code and error, raise
        ``MailChimpException``. This allows callers to wrap method calls in a
        try/except clause and work with a single exception type for any
        error returned by MailChimp.
        """
        # case: response is not a dict so it cannot be an error response
        if not isinstance(response, dict):
            return
        # case: response is a dict and may be an error response
        # We need a status.
        if 'status' not in response:
            return
        status = response['status']
        if 'detail' in response:
            exc = MailChimpException(
                status, response['detail'], response.get('errors')
            )
            logger.warn(exc)
            raise exc
        # Is status a number?
        try:
            status = int(status)
        except ValueError:
            return
        if status >= 400:
            # Maybe we should do this from 300 onwards, but that would include
            # redirects, which might be fine.  In my case I have seen 503,
            # which apparently means we are blocked. Hopefully temporarily.
            # {'type': 'akamai_error_message', 'title': 'akamai_503', 'status': 503,
            #  'ref_no': 'Reference Number: ...'}
            if (
                response.get('type', '') == 'akamai_error_message'
                and 'ref_no' in response
            ):
                exc = AkamaiException(
                    status, response['ref_no'], response.get('title')
                )
            else:
                # Log the full response, as we cannot really parse it.
                logger.warn("Can't properly parse response: %r", response)
                exc = MailChimpException(
                    status, 'no details available', response.get('errors')
                )
            logger.warn(exc)
            raise exc

    def api_request(self, endpoint='', request_type='get', query_params={}, **kwargs):
        """ construct the request and do a get/post.
        """
        if not self.api_root:
            return []
        headers = {'content-type': 'application/json'}
        url = urlparse.urljoin(self.api_root, endpoint)
        if query_params:
            url = "{}?{}".format(url, urlencode(query_params))

        # we provide a json structure with the parameters.
        payload = json.dumps(kwargs)

        assert request_type in ['get', 'post', 'put', 'delete', 'patch']
        request_method = getattr(requests, request_type)

        try:
            resp = request_method(
                url,
                auth=('apikey', self.apikey),
                data=payload,
                headers=headers,
            )
        except Exception as e:
            raise PostRequestError(e)
        decoded = self._deserialize_response(resp.text)
        return decoded

    def lists(self):
        """ API list call. To lower the amount of requests we use a cache
            If the cache is empty we fetch it once from the Mailchimp API.
        """
        self.initialize()
        cache = self.registry.get(self.key_lists, _marker)
        if cache and cache is not _marker:
            return cache
        return self._lists()

    def _lists(self):
        """ The actual API call for lists. """
        count = self.settings.max_lists_number
        try:
            # lists returns a dict with 'total' and 'data'. we just need data
            response = self.api_request('lists', query_params={"count": count})
        except PostRequestError:
            return []
        if 'lists' in response:
            return response['lists']
        return []

    def default_list_id(self):
        """ returns the first item in the list """
        self.initialize()
        if self.settings.default_list:
            return self.settings.default_list
        lists = self.lists()
        if len(lists) > 0:
            return lists[0]['id']

    def groups(self, list_id=None):
        """ API call for interest-categories. This is also cached. """
        if not list_id:
            return
        self.initialize()
        cache = self.registry.get(self.key_groups, _marker)
        if cache and cache is not _marker:
            groups = cache.get(list_id, _marker)
            if groups and groups is not _marker:
                return groups
        return self._groupings(list_id)

    def _groupings(self, list_id=None):
        # Return combined list of interest categories with their group options.
        # We only support one interest category per list.
        # We take the top one.
        categories = self._interest_categories(list_id)
        if not categories or not categories['categories']:
            return categories
        # We could search for the category with the highest display_order key,
        # but for a test list I made, this was zero in both interest
        # categories...  So just take the first one.
        category = categories['categories'][0]
        category_id = category.get('id')
        interests = self._interests(list_id, category_id)
        categories['interests'] = interests.get('interests', [])
        return categories

    def _interest_categories(self, list_id=None):
        """API call to Mailchimp to get interest categories.
        """
        if not list_id:
            return
        url = 'lists/' + list_id + '/interest-categories'
        try:
            return self.api_request(url)
        except MailChimpException:
            raise

    def _interests(self, list_id=None, interest_category_id=None):
        """Actual API call to Mailchimp service to get the group options.

        In api 1.3 this was given immediately with the list of interest
        categories.  In api 3.0 we need to get each interest category
        and ask for its interests/groups.
        """
        if not list_id or not interest_category_id:
            return
        url = 'lists/{0}/interest-categories/{1}/interests'.format(
            list_id, interest_category_id
        )
        try:
            return self.api_request(url)
        except MailChimpException:
            raise

    def subscribe(self, list_id, email_address, email_type, **kwargs):
        """ API call to subscribe a member to a list. """
        self.initialize()
        opt_in_status = 'subscribed'
        if self.settings.double_optin:
            opt_in_status = 'pending'
        if not email_type:
            email_type = self.settings.email_type
        endpoint = 'lists/' + list_id + '/members'
        try:
            response = self.api_request(
                endpoint,
                request_type='post',
                status=opt_in_status,
                email_address=email_address,
                email_type=email_type,
                **kwargs
            )
        except MailChimpException:
            raise
        except Exception as e:
            raise PostRequestError(e)
        logger.info(
            "Subscribed %s to list with id: %s." % (email_address, list_id)
        )
        logger.debug(
            "Subscribed %s to list with id: %s.\n\n %s"
            % (email_address, list_id, response)
        )
        return response

    def get_email_hash(self, email_address):
        if isinstance(email_address, str):
            email_address = email_address.encode("utf-8")
        return hashlib.md5(email_address).hexdigest()

    def update_subscriber(self, list_id, email_address, **kwargs):
        """API call to update a member's data"""
        self.initialize()

        email_hash = self.get_email_hash(email_address)
        endpoint = 'lists/' + list_id + '/members/' + email_hash
        try:
            response = self.api_request(
                endpoint, request_type='patch', **kwargs
            )
        except MailChimpException:
            raise
        except Exception as e:
            raise PostRequestError(e)
        logger.info(
            "Updated %s in list with id: %s." % (email_address, list_id)
        )
        logger.debug(
            "updated %s in list with id: %s.\n\n %s"
            % (email_address, list_id, response)
        )
        return response

    def get_subscriber(self, list_id, email_address, **kwargs):
        """ API call to unsubscribe a member completely from a list."""
        self.initialize()

        email_hash = self.get_email_hash(email_address)
        endpoint = 'lists/' + list_id + '/members/' + email_hash
        try:
            response = self.api_request(endpoint, request_type='get', **kwargs)
        except MailChimpException:
            raise
        except Exception as e:
            raise PostRequestError(e)
        return response

    def add_note_to_subscriber(self, list_id, email_address, note):
        """ API call to unsubscribe a member completely from a list."""
        self.initialize()

        email_hash = self.get_email_hash(email_address)
        endpoint = 'lists/' + list_id + '/members/' + email_hash + '/notes'
        try:
            response = self.api_request(
                endpoint, request_type='post', note=note
            )
        except MailChimpException:
            raise
        except Exception as e:
            raise PostRequestError(e)
        logger.info(
            "Added note for %s in list with id: %s." % (email_address, list_id)
        )
        logger.debug(
            "Added note for %s in list with id: %s.\n\n %s"
            % (email_address, list_id, response)
        )
        return response

    def account(self):
        """ Get account details. This is cached as well """
        self.initialize()
        cache = self.registry.get(self.key_account, _marker)
        if cache and cache is not _marker:
            return cache
        return self._account()

    def _account(self):
        """ Actual API call to mailchimps api root.
        """
        try:
            return self.api_request()
        except MailChimpException:
            logger.exception("Exception getting account details.")
            return None

    def updateCache(self):
        """ Update cache of data from the mailchimp server.  First reset
            our mailchimp object, as the user may have picked a
            different api key.  Alternatively, compare
            self.settings.api_key and self.mailchimp.apikey.
            Connecting will recreate the mailchimp object.
        """
        self.initialize()
        if not self.settings.api_key:
            return
        # Note that we must call the _underscore methods.  These
        # bypass the cache and go directly to mailchimp, so we are
        # certain to have up to date information.
        account = self._account()
        groups = {}
        lists = self._lists()
        for mailchimp_list in lists:
            list_id = mailchimp_list['id']
            groups[list_id] = self._groupings(list_id=list_id)

        # Now save this to the registry, but only if there are
        # changes, otherwise we would do a commit every time we look
        # at the control panel.
        if type(account) is dict:
            if self.registry[self.key_account] != account:
                self.registry[self.key_account] = account
        if type(groups) is dict:
            if self.registry[self.key_groups] != groups:
                self.registry[self.key_groups] = groups
        if type(lists) is list:
            lists = tuple(lists)
            if self.registry[self.key_lists] != lists:
                # Note that unfortunately this happens far too often.
                # In the 'subscribe_url_long' key of an item you can
                # easily first see:
                # 'http://edata.us3.list-manage.com/subscribe?u=abc123',
                # and a second later:
                # 'http://edata.us3.list-manage1.com/subscribe?u=abc123',
                self.registry[self.key_lists] = lists



# -*- coding: utf-8 -*-
from collective.mailchimp.browser import portlet as mailchimp
from collective.mailchimp.testing import (
    COLLECTIVE_MAILCHIMP_INTEGRATION_TESTING,
)
from plone.app.portlets.storage import PortletAssignmentMapping
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.portlets.interfaces import IPortletAssignment
from plone.portlets.interfaces import IPortletDataProvider
from plone.portlets.interfaces import IPortletManager
from plone.portlets.interfaces import IPortletRenderer
from plone.portlets.interfaces import IPortletType
from plone.testing.z2 import Browser
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.site.hooks import setHooks

import unittest


class TestPortlet(unittest.TestCase):

    layer = COLLECTIVE_MAILCHIMP_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        setHooks()

    def testPortletTypeRegistered(self):
        portlet = getUtility(IPortletType, name='portlet.MailChimp')
        self.assertEqual(portlet.addview, 'portlet.MailChimp')

    def testInterfaces(self):
        portlet = mailchimp.Assignment(name="foo")
        self.assertTrue(IPortletAssignment.providedBy(portlet))
        self.assertTrue(IPortletDataProvider.providedBy(portlet.data))

    def testInvokeAddview(self):
        portlet = getUtility(IPortletType, name='portlet.MailChimp')
        mapping = self.portal.restrictedTraverse(
            '++contextportlets++plone.leftcolumn'
        )
        for m in mapping.keys():
            del mapping[m]
        addview = mapping.restrictedTraverse('+/' + portlet.addview)
        addview.createAndAdd(data={})

        self.assertEqual(len(mapping), 1)
        self.assertTrue(isinstance(mapping.values()[0], mailchimp.Assignment))

    def testInvokeEditView(self):
        mapping = PortletAssignmentMapping()
        request = self.portal.REQUEST

        mapping['foo'] = mailchimp.Assignment(name="foo")
        editview = getMultiAdapter((mapping['foo'], request), name='edit')
        self.assertTrue(isinstance(editview, mailchimp.EditForm))

    def testRenderer(self):
        context = self.portal
        request = self.portal.REQUEST
        view = self.portal.restrictedTraverse('@@plone')
        manager = getUtility(
            IPortletManager, name='plone.leftcolumn', context=self.portal
        )
        assignment = mailchimp.Assignment(name="foo")

        renderer = getMultiAdapter(
            (context, request, view, manager, assignment), IPortletRenderer
        )
        self.assertTrue(isinstance(renderer, mailchimp.Renderer))


class TestRenderer(unittest.TestCase):

    layer = COLLECTIVE_MAILCHIMP_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        setHooks()
        # Make sure News Items use simple_publication_workflow
        self.portal.portal_workflow.setChainForPortalTypes(
            ['News Item'], ['simple_publication_workflow']
        )

    def renderer(
        self,
        context=None,
        request=None,
        view=None,
        manager=None,
        assignment=None,
    ):
        context = context or self.portal
        request = request or self.portal.REQUEST
        view = view or self.portal.restrictedTraverse('@@plone')
        manager = manager or getUtility(
            IPortletManager, name='plone.leftcolumn', context=self.portal
        )
        assignment = assignment or mailchimp.Assignment(
            template='portlet_recent', macro='portlet'
        )

        return getMultiAdapter(
            (context, request, view, manager, assignment), IPortletRenderer
        )


class TestPortletIntegration(unittest.TestCase):

    layer = COLLECTIVE_MAILCHIMP_INTEGRATION_TESTING

    def setUp(self):
        app = self.layer['app']
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        self.portal_url = self.portal.absolute_url()

        self.browser = Browser(app)
        self.browser.handleErrors = False
        self.browser.addHeader(
            'Authorization',
            'Basic %s:%s' % (SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
        )

    def test_add_portlet_form(self):
        self.browser.open(
            self.portal_url
            + "/++contextportlets++plone.leftcolumn/+/portlet.MailChimp"
        )

        self.assertTrue("Add MailChimp Portlet" in self.browser.contents)
        self.assertTrue("Title" in self.browser.contents)
        self.assertTrue("Available lists" in self.browser.contents)
        self.assertTrue("ACME Newsletter" in self.browser.contents)
        self.assertTrue("ACME Newsletter 2" in self.browser.contents)

    def test_add_portlet(self):
        self.browser.open(
            self.portal_url
            + "/++contextportlets++plone.leftcolumn/+/portlet.MailChimp"
        )
        self.browser.getControl("Title").value = "ACME Newsletter Portlet"
        self.browser.getControl(
            name="form.widgets.available_lists:list", index=0
        ).value = ["f6257645gs"]
        self.browser.getControl("Save").click()

        self.assertEqual(
            self.browser.url, self.portal_url + '/@@manage-portlets'
        )
        self.assertTrue("Hide" in self.browser.contents)
        self.assertTrue("MailChimp" in self.browser.contents)

        self.browser.open(self.portal_url)
        self.assertTrue("ACME Newsletter Portlet" in self.browser.contents)
        self.assertTrue("Email address" in self.browser.contents)

    def test_edit_portlet(self):
        # Create portlet
        self.browser.open(
            self.portal_url
            + "/++contextportlets++plone.leftcolumn/+/portlet.MailChimp"
        )
        self.browser.getControl("Title").value = "ACME Newsletter Portlet"
        self.browser.getControl(
            name="form.widgets.available_lists:list", index=0
        ).value = ["f6257645gs"]
        self.browser.getControl("Save").click()
        # Edit portlet
        self.browser.open(
            self.portal_url
            + "/++contextportlets++plone.leftcolumn/mailchimp/edit"
        )
        self.browser.getControl("Title").value = "Lorem Ipsum"
        self.browser.getControl(
            name="form.widgets.available_lists:list", index=0
        ).value = ["f6267645gs"]
        self.browser.getControl("Save").click()

        self.browser.open(
            self.portal_url
            + "/++contextportlets++plone.leftcolumn/mailchimp/edit"
        )
        self.assertTrue("Lorem Ipsum" in self.browser.contents)
        self.browser.open(self.portal_url)
        self.assertTrue("Lorem Ipsum" in self.browser.contents)


def test_suite():
    return unittest.defaultTestLoader.loadTestsFromName(__name__)



# -*- coding: utf-8 -*-
from collective.mailchimp.browser.newsletter import NewsletterSubscriberForm
from collective.mailchimp.testing import (
    COLLECTIVE_MAILCHIMP_INTEGRATION_TESTING,
)
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.testing.z2 import Browser
from plone.z3cform.fieldsets import extensible
from zope import schema
from zope.component import adapts
from zope.component import provideAdapter
from zope.interface import Interface
from zope.interface import Invalid

import unittest


def validate(value):
    if value is not True:
        raise Invalid(u'Required test field not checked')
    return True


class ITestExtenderSchema(Interface):

    accept = schema.Bool(title=u'accept', required=True, constraint=validate)


class TestExtender(extensible.FormExtender):
    adapts(Interface, Interface, NewsletterSubscriberForm)

    def update(self):
        self.add(ITestExtenderSchema)


class TestNewsletterView(unittest.TestCase):

    layer = COLLECTIVE_MAILCHIMP_INTEGRATION_TESTING

    def setUp(self):
        app = self.layer['app']
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        self.portal_url = self.portal.absolute_url()
        self.browser = Browser(app)
        self.browser.handleErrors = False
        self.browser.addHeader(
            'Authorization',
            'Basic %s:%s' % (SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
        )

    def test_empty_form(self):
        self.browser.open("%s/newsletter" % self.portal_url)
        self.browser.getControl(name="form.buttons.subscribe").click()
        self.assertTrue("Required input is missing." in self.browser.contents)

    def test_form_with_invalid_email_address(self):
        self.browser.open("%s/newsletter" % self.portal_url)
        self.browser.getControl(
            name="form.widgets.email"
        ).value = "Not an email address"
        self.browser.getControl(name="form.buttons.subscribe").click()
        self.assertTrue("Invalid email address" in self.browser.contents)


class TestNewsletterExtender(unittest.TestCase):

    layer = COLLECTIVE_MAILCHIMP_INTEGRATION_TESTING

    def setUp(self):
        app = self.layer['app']
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        self.portal_url = self.portal.absolute_url()
        self.browser = Browser(app)
        self.browser.handleErrors = False
        self.browser.addHeader(
            'Authorization',
            'Basic %s:%s' % (SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
        )
        provideAdapter(factory=TestExtender)

    def test_newsletter_extender(self):
        self.browser.open("%s/newsletter" % self.portal_url)
        self.browser.getControl(
            name="form.widgets.email"
        ).value = "valid@email.com"
        self.browser.getControl(name="form.buttons.subscribe").click()
        self.assertTrue(
            "Required test field not checked" in self.browser.contents
        )


#    def test_form_with_valid_email_address(self):
#        self.browser.open("%s/newsletter" % self.portal_url)
#        self.browser.getControl(name="form.widgets.email").value = \
#            "john@doe.com"
#        self.browser.getControl(name="form.buttons.subscribe").click()

#    def test_form_with_already_subscribed_email_address(self):
#        # Subscribe once
#        self.browser.open("%s/newsletter" % self.portal_url)
#        self.browser.getControl(name="form.widgets.email").value = \
#            "john@doe.com"
#        self.browser.getControl(name="form.buttons.subscribe").click()
#        # Subscribe twice
#        self.browser.open("%s/newsletter" % self.portal_url)
#        self.browser.getControl(name="form.widgets.email").value = \
#            "john@doe.com"
#        self.browser.getControl(name="form.buttons.subscribe").click()

#    def test_form_with_banned_email_address(self):
#        self.browser.open("%s/newsletter" % self.portal_url)
#        self.browser.getControl(name="form.widgets.email").value = \
#            "john@doe.com"
#        self.browser.getControl(name="form.buttons.subscribe").click()


class TestUnsubscribeNewsletterView(unittest.TestCase):

    layer = COLLECTIVE_MAILCHIMP_INTEGRATION_TESTING

    def setUp(self):
        app = self.layer['app']
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        self.portal_url = self.portal.absolute_url()
        self.browser = Browser(app)
        self.browser.handleErrors = False
        self.browser.addHeader(
            'Authorization',
            'Basic %s:%s' % (SITE_OWNER_NAME, SITE_OWNER_PASSWORD),
        )

    def test_empty_form(self):
        self.browser.open("%s/unsubscribe-newsletter" % self.portal_url)
        self.browser.getControl(name="form.buttons.unsubscribe").click()
        self.assertTrue("Required input is missing." in self.browser.contents)

    def test_form_with_email_address(self):
        self.browser.open("%s/unsubscribe-newsletter" % self.portal_url)
        self.browser.getControl(
            name="form.widgets.email"
        ).value = "freddy@freddiesjokes.com"
        unsub_all_checkbox = self.browser.getControl(
            name='form.widgets.unsubscribe:list', index=0
        )
        unsub_all_checkbox.value = ['selected']
        self.browser.getControl(name="form.buttons.unsubscribe").click()

        from collective.mailchimp.locator import requests

        self.assertEqual(
            requests.last_call,
            {
                'endpoint': (
                    u'lists/f6257645gs/members/'
                    u'06f12badc3b5fffc57576822131ded7c'
                ),
                'data': {u'status': u'unsubscribed'},
            },
        )


def test_suite():
    return unittest.defaultTestLoader.loadTestsFromName(__name__)
